@@ -1006,7 +1006,7 @@ tagQueryMatchChildRev <- function(els, func) {
     elParent <- el$parent
     # Walk in reverse to be able to remove all matches in a single pass
     selectedWalkIRev(elParent$children, function(child, childPos) {
-      if (!isTagEnv(el)) return()
+      if (!isTagEnv(child)) return()
       childKey <- child$envKey
       if (elKey == childKey) {
         func(elParent, el, childPos)
@@ -1535,7 +1535,7 @@ tagQueryFindDescendants_ <- function(el, selector, fn) {
   } else if (is.list(el)) {
     # For each item in the list like object, recurse through
     walk(el, tagQueryFindDescendants_, fn = fn, selector = selector)
-  } else if (is.atomic(el) || is.function(el)) {
+  } else if (is.atomic(el) || is.function(el) || is.language(el)) {
     # Can not match on atomics or functions
     return()
   } else {
