@@ -1757,7 +1757,12 @@ includeText <- function(path) {
 #' @rdname include
 #' @export
 includeMarkdown <- function(path) {
-  html <- markdown::markdownToHTML(path, fragment.only=TRUE)
+  # markdown >= v1.3 switched from markdownToHTML() to mark()
+  html <- if (packageVersion("markdown") < "1.3") {
+    markdown::markdownToHTML(path, fragment.only = TRUE)
+  } else {
+    markdown::mark(path, output = NULL)
+  }
   Encoding(html) <- 'UTF-8'
   return(HTML(html))
 }
